"""

"""

CREATE_VERTICES_SAMPLES = {
    "plant": {
        "label": "Planet",
        "properties": {
            "name": "Earth",
            "mass_in_kgs": float(5972000000000000000000000),
            "radius_in_kms": 6371
        }
    },
    "satellite": {
        "label": "Satellite",
        "properties": {
            "name": "Moon",
            "mass_in_kgs": int(73476730900000000000000)
        }
    }
}
