"""

"""


class InvalidVertexException(BaseException):
    pass


class InvalidPayloadException(BaseException):
    pass


class InvalidConnection(BaseException):
    pass


class InvalidQueryArguments(BaseException):
    pass


class EdgeDoesntExist(BaseException):
    pass


class VertexDoesntExist(BaseException):
    pass
